=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-09-06 15:21:50 +0000
+++ bzrlib/knit.py	2006-09-06 20:42:45 +0000
@@ -279,9 +279,10 @@
     stored and retrieved.
     """
 
-    def __init__(self, relpath, transport, file_mode=None, access_mode=None, 
+    def __init__(self, relpath, transport, file_mode=None, access_mode=None,
                  factory=None, basis_knit=DEPRECATED_PARAMETER, delta=True,
-                 create=False, create_parent_dir=False, delay_create=False):
+                 create=False, create_parent_dir=False, delay_create=False,
+                 dir_mode=None):
         """Construct a knit at location specified by relpath.
         
         :param create: If not True, only open an existing knit.
@@ -307,10 +308,12 @@
 
         self._index = _KnitIndex(transport, relpath + INDEX_SUFFIX,
             access_mode, create=create, file_mode=file_mode,
-            create_parent_dir=create_parent_dir, delay_create=delay_create)
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
         self._data = _KnitData(transport, relpath + DATA_SUFFIX,
             access_mode, create=create and not len(self), file_mode=file_mode,
-            create_parent_dir=create_parent_dir, delay_create=delay_create)
+            create_parent_dir=create_parent_dir, delay_create=delay_create,
+            dir_mode=dir_mode)
 
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, 
@@ -956,11 +959,12 @@
     """One of the files used to implement a knit database"""
 
     def __init__(self, transport, filename, mode, file_mode=None,
-                 create_parent_dir=False):
+                 create_parent_dir=False, dir_mode=None):
         self._transport = transport
         self._filename = filename
         self._mode = mode
         self._file_mode = file_mode
+        self._dir_mode = dir_mode
         self._create_parent_dir = create_parent_dir
         self._need_to_create = False
 
@@ -1056,9 +1060,11 @@
                                    index)
 
     def __init__(self, transport, filename, mode, create=False, file_mode=None,
-                 create_parent_dir=False, delay_create=False):
-        _KnitComponentFile.__init__(self, transport, filename, mode, file_mode,
-                                    create_parent_dir=create_parent_dir)
+                 create_parent_dir=False, delay_create=False, dir_mode=None):
+        _KnitComponentFile.__init__(self, transport, filename, mode,
+                                    file_mode=file_mode,
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
         self._cache = {}
         # position in _history is the 'official' index for a revision
         # but the values may have come from a newer entry.
@@ -1268,7 +1274,8 @@
             sio.seek(0)
             self._transport.put_file_non_atomic(self._filename, sio,
                                 create_parent_dir=self._create_parent_dir,
-                                mode=self._file_mode)
+                                mode=self._file_mode,
+                                dir_mode=self._dir_mode)
             self._need_to_create = False
 
         # cache after writing, so that a failed write leads to missing cache
@@ -1322,9 +1329,11 @@
     """Contents of the knit data file"""
 
     def __init__(self, transport, filename, mode, create=False, file_mode=None,
+                 dir_mode=None,
                  create_parent_dir=False, delay_create=False):
         _KnitComponentFile.__init__(self, transport, filename, mode,
                                     file_mode=file_mode,
+                                    dir_mode=dir_mode,
                                     create_parent_dir=create_parent_dir)
         self._checked = False
         # TODO: jam 20060713 conceptually, this could spill to disk
@@ -1388,7 +1397,8 @@
         else:
             self._transport.put_bytes_non_atomic(self._filename, raw_data,
                                    create_parent_dir=self._create_parent_dir,
-                                   mode=self._file_mode)
+                                   mode=self._file_mode,
+                                   dir_mode=self._dir_mode)
             self._need_to_create = False
             return 0
         
@@ -1402,7 +1412,8 @@
         else:
             self._transport.put_file_non_atomic(self._filename, sio,
                                create_parent_dir=self._create_parent_dir,
-                               mode=self._file_mode)
+                               mode=self._file_mode,
+                               dir_mode=self._dir_mode)
             self._need_to_create = False
             start_pos = 0
         if self._do_cache:

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-09-06 14:37:18 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-09-06 20:42:45 +0000
@@ -243,6 +243,18 @@
         umask = osutils.get_umask()
         t.put_bytes_non_atomic('nomode', 'test text\n', mode=None)
         self.assertTransportMode(t, 'nomode', 0666 & ~umask)
+
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        t.put_bytes_non_atomic('dir700/mode664', 'test text\n', mode=0664,
+                               dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_bytes_non_atomic('dir770/mode664', 'test text\n', mode=0664,
+                               dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_bytes_non_atomic('dir777/mode664', 'test text\n', mode=0664,
+                               dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
         
     def test_put_file(self):
         t = self.get_transport()
@@ -352,6 +364,19 @@
         t.put_file_non_atomic('nomode', StringIO('test text\n'), mode=None)
         self.assertTransportMode(t, 'nomode', 0666 & ~umask)
         
+        # We should also be able to set the mode for a parent directory
+        # when it is created
+        sio = StringIO()
+        t.put_file_non_atomic('dir700/mode664', sio, mode=0664,
+                              dir_mode=0700, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir700', 0700)
+        t.put_file_non_atomic('dir770/mode664', sio, mode=0664,
+                              dir_mode=0770, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir770', 0770)
+        t.put_file_non_atomic('dir777/mode664', sio, mode=0664,
+                              dir_mode=0777, create_parent_dir=True)
+        self.assertTransportMode(t, 'dir777', 0777)
+
     def test_put_multi(self):
         t = self.get_transport()
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-09-06 15:21:50 +0000
+++ bzrlib/transport/__init__.py	2006-09-06 20:42:45 +0000
@@ -526,7 +526,8 @@
         return self.put_file(relpath, StringIO(bytes), mode=mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False,
+                             dir_mode=None):
         """Copy the string into the target location.
 
         This function is not strictly safe to use. See 
@@ -540,11 +541,13 @@
         :param create_parent_dir: If we cannot create the target file because
                         the parent directory does not exist, go ahead and
                         create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
         """
         assert isinstance(bytes, str), \
             'bytes must be a plain string, not %s' % type(bytes)
         self.put_file_non_atomic(relpath, StringIO(bytes), mode=mode,
-                                 create_parent_dir=create_parent_dir)
+                                 create_parent_dir=create_parent_dir,
+                                 dir_mode=dir_mode)
 
     def put_file(self, relpath, f, mode=None):
         """Copy the file-like object into the location.
@@ -565,7 +568,8 @@
         #raise NotImplementedError(self.put_file)
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -580,6 +584,7 @@
         :param create_parent_dir: If we cannot create the target file because
                         the parent directory does not exist, go ahead and
                         create it, and then try again.
+        :param dir_mode: Possible access permissions for new directories.
         """
         # Default implementation just does an atomic put.
         try:
@@ -589,7 +594,7 @@
                 raise
             parent_dir = osutils.dirname(relpath)
             if parent_dir:
-                self.mkdir(parent_dir)
+                self.mkdir(parent_dir, mode=dir_mode)
                 return self.put_file(relpath, f, mode=mode)
 
     @deprecated_method(zero_eleven)

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2006-09-06 02:57:51 +0000
+++ bzrlib/transport/local.py	2006-09-06 20:42:45 +0000
@@ -177,7 +177,8 @@
 
     def _put_non_atomic_helper(self, relpath, writer,
                                mode=None,
-                               create_parent_dir=False):
+                               create_parent_dir=False,
+                               dir_mode=None):
         """Common functionality information for the put_*_non_atomic.
 
         This tracks all the create_parent_dir stuff.
@@ -206,10 +207,7 @@
             parent_dir = os.path.dirname(abspath)
             if not parent_dir:
                 self._translate_error(e, relpath)
-            try:
-                os.mkdir(parent_dir)
-            except (IOError, OSError), e:
-                self._translate_error(e, relpath)
+            self._mkdir(parent_dir, mode=dir_mode)
             # We created the parent directory, lets try to open the
             # file again
             try:
@@ -227,7 +225,8 @@
             os.close(fd)
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -246,14 +245,16 @@
         def writer(fd):
             self._pump_to_fd(f, fd)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False, dir_mode=None):
         def writer(fd):
             os.write(fd, bytes)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def iter_files_recursive(self):
         """Iter the relative paths of files in the transports sub-tree."""
@@ -267,23 +268,25 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath, mode=None):
-        """Create a directory at the given path."""
-        path = relpath
+    def _mkdir(self, abspath, mode=None):
+        """Create a real directory, filtering through mode"""
+        if mode is None:
+            # os.mkdir() will filter through umask
+            local_mode = 0777
+        else:
+            local_mode = mode
         try:
-            if mode is None:
-                # os.mkdir() will filter through umask
-                local_mode = 0777
-            else:
-                local_mode = mode
-            path = self._abspath(relpath)
-            os.mkdir(path, local_mode)
+            os.mkdir(abspath, local_mode)
             if mode is not None:
                 # It is probably faster to just do the chmod, rather than
                 # doing a stat, and then trying to compare
-                os.chmod(path, mode)
+                os.chmod(abspath, mode)
         except (IOError, OSError),e:
-            self._translate_error(e, path)
+            self._translate_error(e, abspath)
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        self._mkdir(self._abspath(relpath), mode=mode)
 
     def _get_append_file(self, relpath, mode=None):
         """Call os.open() for the given relpath"""

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-09-06 15:21:50 +0000
+++ bzrlib/transport/sftp.py	2006-09-06 20:42:45 +0000
@@ -426,7 +426,8 @@
             raise
 
     def _put_non_atomic_helper(self, relpath, writer, mode=None,
-                               create_parent_dir=False):
+                               create_parent_dir=False,
+                               dir_mode=None):
         abspath = self._remote_path(relpath)
 
         # TODO: jam 20060816 paramiko doesn't publicly expose a way to
@@ -467,12 +468,15 @@
             parent_dir = os.path.dirname(abspath)
             try:
                 self._sftp.mkdir(parent_dir)
+                if dir_mode is not None:
+                    self._sftp.chmod(path, mode=dir_mode)
             except (paramiko.SSHException, IOError), e:
                 self._translate_io_exception(e, abspath, ': unable to open')
             _open_and_write_file()
 
     def put_file_non_atomic(self, relpath, f, mode=None,
-                            create_parent_dir=False):
+                            create_parent_dir=False,
+                            dir_mode=None):
         """Copy the file-like object into the target location.
 
         This function is not strictly safe to use. It is only meant to
@@ -491,14 +495,17 @@
         def writer(fout):
             self._pump(f, fout)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def put_bytes_non_atomic(self, relpath, bytes, mode=None,
-                             create_parent_dir=False):
+                             create_parent_dir=False,
+                             dir_mode=None):
         def writer(fout):
             fout.write(bytes)
         self._put_non_atomic_helper(relpath, writer, mode=mode,
-                                    create_parent_dir=create_parent_dir)
+                                    create_parent_dir=create_parent_dir,
+                                    dir_mode=dir_mode)
 
     def iter_files_recursive(self):
         """Walk the relative paths of all files in this transport."""

